clear all
clc;
%% Test case 1 for simulation

% generate data
P_d_min = 1; % minimum active demand
P_d_max = 3; % max active demand
Q_d_max = 0.6*P_d_max; % Q_demand = 0.2*P_demand;
PV_pene_level = 0.5; % penetration level of renewable units+
lambda = 6.6; % real power price
kappa =  0.75*lambda; % reactive price

%% Set parameters
N = 250; % number of users/nodes 
S_max = 2.2*ones(N,1); % nameplate capacity of DU units

% P_d = P_d_min + (P_d_max-P_d_min)*rand(N,1); % real power demand
% Q_d = Q_d_max*rand(N,1); % reactive demand
% P_g_max = (0.75 + (1-0.75).*rand(N,1)).*S_max; % maximum real power generation capcity of renewable units from 0.75 -> max
% % user_PV = randperm(N,floor(PV_pene_level*N)); % determine array of users equipped with DG units
% % save demand data into mat file
% save('Input\P_demand.mat','P_d');
% save('Input\Q_demand.mat','Q_d');
% save('Input\Pg_max.mat','P_g_max');
% save('userPV.mat','user_PV');


load('Input\P_demand.mat','P_d');
load('Input\Q_demand.mat','Q_d');
load('Input\Pg_max.mat','P_g_max');
load('Input\userPV_50.mat','user_PV');

%%define variable for active and reactive power generation
P_g_nocompen = zeros(N,1);
P_g = zeros(N,1);
Q_g = zeros(N,1);
z_n_cost = zeros(N,1); % cost incurred to users due to reduce real power
seq_welfare_allocation = zeros(N,1); % social welfare of each user
z_n_seq = zeros(N,1); % reimbursement in sequential bargaining
z_n_conc = zeros(N,1); % reimbursemnet in concurrent bargaining
conc_welfare_allocation = zeros(N,1); % social welfare allocated for each user in concurrent bargaining
% total demand of real and reactive power demand before compensation
P_d_total = sum(P_d);
Q_d_total = sum(Q_d);

%% condense variables for only PV users
P_g_nocompen_densed = zeros(length(user_PV),1);
P_g_densed = zeros(length(user_PV),1);
Q_g_densed = zeros(length(user_PV),1);
z_n_cost_densed = zeros(length(user_PV),1); % cost incurred to users due to reduce real power
seq_welfare_allocation_densed = zeros(length(user_PV),1); % social welfare allocation to each user in sequentil bargaining
z_n_seq_densed = zeros(length(user_PV),1); % reimbursement in sequential bargaining
z_n_conc_densed = zeros(length(user_PV),1); % reimbursemnet in concurrent bargaining
conc_welfare_allocation_densed = zeros(length(user_PV),1); % social welfare allocated for each user in concurrent bargaining

% get demand data for user PV only, this is used for plot figure, not
% necessary in running simulation
P_d_densed = zeros(length(user_PV),1);
Q_d_densed = zeros(length(user_PV),1);

%% optimal active and reactive power generation 
for i=1:length(user_PV)
    % acive power generation without reactive power compensation
    P_g_nocompen(user_PV(i)) = min(P_d(user_PV(i)),P_g_max(user_PV(i)));
    P_g_nocompen_densed(i) = P_g_nocompen(user_PV(i));
    
    % real and reactive power generation with compensation
    if (P_g_nocompen(user_PV(i))^2 + Q_d(user_PV(i))^2 <= S_max(user_PV(i))^2) % if the total demand less than generation capacity
        P_g(user_PV(i)) = P_g_nocompen(user_PV(i));
        Q_g(user_PV(i)) = Q_d(user_PV(i));
    else        
    P_g(user_PV(i)) = min(P_g_nocompen(user_PV(i)), lambda*S_max(user_PV(i))/sqrt(lambda^2 + kappa^2) ); % real power generation  
        
    Q_g(user_PV(i)) = min( Q_d(user_PV(i)), kappa*S_max(user_PV(i))/sqrt(lambda^2 + kappa^2) ); % reactive power generation
        
    end
    P_g_densed(i) = P_g(user_PV(i));
    Q_g_densed(i) = Q_g(user_PV(i));
    
    % calculate reimbursement 
    % calculate cost incurred by reducing real power
    z_n_cost(user_PV(i)) = lambda*( P_g_nocompen(user_PV(i)) - P_g(user_PV(i)) );
    z_n_cost_densed(i) = z_n_cost(user_PV(i));
    
    seq_welfare_allocation(user_PV(i)) =  kappa*Q_g(user_PV(i)) - lambda*( P_g_nocompen(user_PV(i)) - P_g(user_PV(i)) );
    seq_welfare_allocation_densed(i) = seq_welfare_allocation(user_PV(i));
    
    z_n_seq(user_PV(i)) = z_n_cost(user_PV(i)) + 0.5*seq_welfare_allocation(user_PV(i));
    z_n_seq_densed(i) = z_n_seq(user_PV(i));
    
    % copy demand data to variable P_d_densed and Q_d_densed, which is
    % using for plotting
    P_d_densed(i) = P_d(user_PV(i));
    Q_d_densed(i) = Q_d(user_PV(i));
end

% calculate reimubursement for concurrent bargaining
for i=1:length(user_PV)
    conc_welfare_allocation(user_PV(i)) = sum(seq_welfare_allocation)/( length(user_PV) + 1 );
    conc_welfare_allocation_densed(i) = conc_welfare_allocation(user_PV(i));
    
    z_n_conc(user_PV(i)) = z_n_cost(user_PV(i)) + sum(seq_welfare_allocation)/( length(user_PV) + 1 );
    z_n_conc_densed(i) =  z_n_conc(user_PV(i));
end

% determnine the total power dispath after compensation

P_inj = sum(P_d - P_g);
Q_inj = sum(Q_d - Q_g);

% calculate power factor
PF_orig = P_d_total/sqrt(P_d_total^2 + Q_d_total^2);
PF_new = P_inj/sqrt(P_inj^2 + Q_inj^2);


%% write to excel file 
xlswrite('Output\Real_P_Nocomp.xlsx', P_g_nocompen_densed);
xlswrite('Output\Real_P_Comp.xlsx', P_g_densed);
xlswrite('Output\Reactive_Q_Comp.xlsx', Q_g_densed);
xlswrite('Output\Cost_Reduce_P.xlsx', z_n_cost_densed);
xlswrite('Output\Seq_Welfare_Allocation.xlsx', 0.5.*seq_welfare_allocation_densed);
xlswrite('Output\Seq_Reimbursement.xlsx', z_n_seq_densed);
xlswrite('Output\Conc_Welfare_Allocation.xlsx', conc_welfare_allocation_densed);
xlswrite('Output\Conc_Reimursement.xlsx', z_n_conc_densed);
xlswrite('Output\P_demand_densed.xlsx', P_d_densed);
xlswrite('Output\Q_demand_densed.xlsx', Q_d_densed);